# ------------------------------------------------------------------------------

# Processamento dos dados biológicos

# ------------------------------------------------------------------------------


# ================================ BIBLIOTECAS =================================
 
require(affy)
# require(hgu133a.db) 
require(u133agbcdf)
require(splines)

# ================================ DEPENDENCIAS ================================ 

source("simulation.r")

# Diretorio com os arquivos .cel retirados do cArray Data Portal, id jacob-00182.
setwd("~/IME/IC/Dados/caArray_jacob-00182_files-AFFYMETRIX_CEL") 

# Tabela com os dados normalizados pelo justRMA a partor dos arquivos .cel
table <- read.table("~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados")


# ============================= FUNCOES AUXILIARES ============================= 

is.stage1 <- function(p, i) {
    T = strsplit(as.character(p[i, "PATHOLOGIC_T_STAGE"]), ",")[[1]][1]
    if (is.na(T)) return(FALSE)
    if (strsplit(as.character(p[i, "PATHOLOGIC_N_STAGE"]), ",")[[1]][1] == "N0") {    
        if (T == "T1" || T == "T2")
            return(TRUE)
    }
    return(FALSE)
}


# ============================ FUNCOES DA INTERFACE ============================ 

normalize.data <- function() {
    files <- list.celfiles()
    patients <- vector()
    p <- read.csv("~/IME/IC/R/patients.csv")
    n = nrow(p)
    for (i in 1:n)
        patients[paste(as.character(p[i, "MICROARRAY"]), "CEL", sep=".")] <- as.character(p[i, "PATIENT_ID"])
    patients = patients[files]
    patients = patients[which(!is.na(patients))]
    files = names(patients)
    table = justRMA(filenames = files, cdfname="u133agbcdf")
    table = exprs(table)
    names = colnames(table)
    colnames(table) = patients[names]
    write.table(table, "~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - log2")
}

convert.data <- function() {
    table = read.table("~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - log2")
    log2exp = log2(exp(1))
    result = exp(table/log2exp)
    write.table(result, "~/Dropbox/Suzana/Dados/Dados normalizados (justRMA)")
}

select.genes <- function() {
    table = read.table("~/Dropbox/Suzana/Dados/Dados normalizados (justRMA)")
    symbols <- readLines("~/IME/IC/R/selected genes u133agb")
    db <- as.matrix(read.table("~/IME/IC/R/Pacotes/U133Agb.db", sep=","))
    names <- rownames(table)
    db <- db[which(db[,1] %in% names),]
    map <- db[, 1]
    names(map) <- db[,3] 
    map = map[symbols]
    genes <- map[which(!is.na(map))]
    result <- table[genes,]
    rownames(result) <- names(genes)
    write.table(result, "~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados")
}

select.genes.control <- function() {
    table = read.table("~/Dropbox/Suzana/Dados/Dados normalizados (justRMA)")
    db <- as.matrix(read.table("~/IME/IC/R/Pacotes/U133Agb.db", sep=","))
    names <- rownames(table)
    genes <- append(db[which(db[,5] == "control"),1], db[which(db[,3] == "WNT5A"),1])
    names(genes) <- append(genes[1:(length(genes) - 1)], "WNT5A")
    result <- table[genes,]
    rownames(result) <- names(genes)
    write.table(result, "~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes de controle")
}


test.genes.without.outliers.control <- function(method, legend,  sites) {
    test.genes.without.outliers(method, paste(legends[i], "Genes de controle", sep=" - "), sites, input="~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes de controle")
}

test.genes <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    files <- list.celfiles()
    patients <- vector()
    # sites = c("DFCI", "HLM", "MI", "MSKCC")
    p <- read.csv("~/IME/IC/R/patients.csv")
    n = nrow(p)
    for (i in 1:n)
        if (is.stage1(p, i))
            patients[paste(as.character(p[i, "MICROARRAY"]), "CEL", sep=".")] <- as.character(p[i, "PATIENT_ID"])
    patients = patients[files]
    patients = patients[which(!is.na(patients))]
    names(patients) = patients
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients))
         patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
    
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    if (length(sites) > 1)
        for (i in 2:length(sites))
            label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            pvalue = as.numeric(method(WNT5A, as.numeric(table[gene, samesite]))["p-valor"]) 
            #z = qnorm(pvalue/2, lower.tail=FALSE)
            #sum = sum + z
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/Dropbox/Suzana/Dados/P-valores/Nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/Dropbox/Suzana/Dados/P-valores/Ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}



test.genes.without.outliers <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    files <- list.celfiles()
    patients <- vector()
    # sites = c("DFCI", "HLM", "MI", "MSKCC")
    p <- read.csv("~/IME/IC/R/patients.csv")
    n = nrow(p)
    for (i in 1:n)
        if (is.stage1(p, i))
            patients[paste(as.character(p[i, "MICROARRAY"]), "CEL", sep=".")] <- as.character(p[i, "PATIENT_ID"])
    patients = patients[files]
    patients = patients[which(!is.na(patients))]
    names(patients) = patients
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients))
         patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
    
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    if (length(sites) > 1)
        for (i in 2:length(sites))
            label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            y = as.numeric(table[gene, samesite])
            pdf(".pdf")
            b = boxplot(y)
            dev.off()
            index = which(!(y %in% b$out))
            pvalue = as.numeric(method(WNT5A[index], y[index])["p-valor"])
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}

test.genes.linux <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/private/ime-7o-semestre/IC/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    p <- read.csv("~/private/ime-7o-semestre/IC/R/patients.csv")
    patients <- read.table("~/private/ime-7o-semestre/IC/R/patients")
    patients = patients[,1]
    names(patients) = patients
    # sites = c("DFCI", "HLM", "MI", "MSKCC")
    n = nrow(p)
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients)) {
             if (is.stage1(p, i))
                patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
              else patients[as.character(p[i, "PATIENT_ID"])] <- NA
        }
    patients <- patients[which(!is.na(patients))]
    
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    for (i in 2:length(sites))
        label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            pvalue = as.numeric(method(WNT5A, as.numeric(table[gene, samesite]))["p-valor"]) 
            #z = qnorm(pvalue/2, lower.tail=FALSE)
            #sum = sum + z
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/private/ime-7o-semestre/IC/Dados/P-valores/Nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/private/ime-7o-semestre/IC/Dados/P-valores/Ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}



test.genes.without.outliers.linux <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/private/ime-7o-semestre/IC/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    p <- read.csv("~/private/ime-7o-semestre/IC/R/patients.csv")
    patients <- read.table("~/private/ime-7o-semestre/IC/R/patients")
    patients = patients[,1]
    names(patients) = patients
    n = nrow(p)
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients)) {
             if (is.stage1(p, i))
                patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
              else patients[as.character(p[i, "PATIENT_ID"])] <- NA
        }
    patients <- patients[which(!is.na(patients))]
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    if (length(sites) > 1)
        for (i in 2:length(sites))
            label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            y = as.numeric(table[gene, samesite])
            pdf(".pdf")
            b = boxplot(y)
            dev.off()
            index = which(!(y %in% b$out))
            pvalue = as.numeric(method(WNT5A[index], y[index])["p-valor"])
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/private/ime-7o-semestre/IC/Dados/P-valores/Sem outliers - nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/private/ime-7o-semestre/IC/Dados/P-valores/Sem outliers - ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}


test.genes.vision <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    p <- read.csv("~/Dropbox/Suzana/R/patients.csv")
    patients <- read.table("~/Dropbox/Suzana/R/patients")
    patients = patients[,1]
    names(patients) = patients
    # sites = c("DFCI", "HLM", "MI", "MSKCC")
    n = nrow(p)
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients)) {
             if (is.stage1(p, i))
                patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
              else patients[as.character(p[i, "PATIENT_ID"])] <- NA
        }
    patients <- patients[which(!is.na(patients))]
    
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    for (i in 2:length(sites))
        label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            pvalue = as.numeric(method(WNT5A, as.numeric(table[gene, samesite]))["p-valor"]) 
            #z = qnorm(pvalue/2, lower.tail=FALSE)
            #sum = sum + z
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/Dropbox/Suzana/Dados/P-valores/vision/Nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/Dropbox/Suzana/Dados/P-valores/vision/Ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}


test.genes.without.outliers.vision <- function(method, legend, sites = c("HLM", "MI", "MSKCC"), input="~/Dropbox/Suzana/Dados/Dados normalizados (justRMA) - genes selecionados") {
    table <- read.table(input)
    names <- rownames(table)
    table <- as.matrix(table)
    p <- read.csv("~/Dropbox/Suzana/R/patients.csv")
    patients <- read.table("~/Dropbox/Suzana/R/patients")
    patients = patients[,1]
    names(patients) = patients
    n = nrow(p)
    for (i in 1:n)
        if(as.character(p[i, "PATIENT_ID"]) %in% names(patients)) {
             if (is.stage1(p, i))
                patients[as.character(p[i, "PATIENT_ID"])] <- as.character(p[i, "SITE"])
              else patients[as.character(p[i, "PATIENT_ID"])] <- NA
        }
    patients <- patients[which(!is.na(patients))]
    WNT5A <- as.numeric(table["WNT5A",])
    genes <- names[which(names != "WNT5A")]
    pvalues <- vector()

    label = sites[1]
    if (length(sites) > 1)
        for (i in 2:length(sites))
            label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")
    i = 0
    for (gene in genes) {
        sum = 0
        for (site in sites) {
            samesite <- paste("X", names(patients[which(patients == site)]), sep="")
            WNT5A <- as.numeric(table["WNT5A", samesite])
            y = as.numeric(table[gene, samesite])
            pdf(".pdf")
            b = boxplot(y)
            dev.off()
            index = which(!(y %in% b$out))
            pvalue = as.numeric(method(WNT5A[index], y[index])["p-valor"])
            sum = sum + log(pvalue)
        }
        i = i + 1
        #pvalues[i] <- 2*pnorm(sum, mean = 0, sd = 1, lower.tail = FALSE)
        pvalues[i] <- 1 - pchisq(-2*sum, 2*length(sites))
        write(pvalues[i], paste("~/Dropbox/Suzana/Dados/P-valores/vision/Sem outliers - nao ajustados - ", 
            paste(legend, "Fisher", sep=" - "), sep=label), append=TRUE)
    }
    pvalues = p.adjust(pvalues, method = "fdr")
    names(pvalues) = genes
    write.table(pvalues, paste("~/Dropbox/Suzana/Dados/P-valores/vision/Sem outliers - ajustados - ",  paste(legend, "Fisher", sep=" - "), sep=label))
}



analyse.associations <- function() {
    HHG = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - HHG - Fisher", header=TRUE)
    Hoeffding = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - Hoeffding - Fisher", header=TRUE)
    Dcor = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - Dcor - Fisher", header=TRUE)
    Kendall = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - Kendall - Fisher", header=TRUE)
    Pearson = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - Pearson - Fisher", header=TRUE)
    Spearman = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - Spearman - Fisher", header=TRUE)
    CIM = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - CIM - Fisher", header=TRUE)
    IM = read.table("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - ajustados - HLM,MI,MSKCC - IM - Fisher", header=TRUE)


    HHG = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - HHG - Fisher"))
    Hoeffding = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - Hoeffding - Fisher"))
    Dcor = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - Dcor - Fisher"))
    Kendall = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - Kendall - Fisher"))
    Pearson = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - Pearson - Fisher"))
    Spearman = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - Spearman - Fisher"))
    CIM = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - CIM - Fisher"))
    IM = as.numeric(readLines("~/Dropbox/Suzana/Dados/P-valores/Sem outliers - nao ajustados - HLM,MI,MSKCC - IM - Fisher"))



    names = rownames(HHG)
    linear = vector()
    mono = vector()
    nmono = vector()
    for (name in names) {
        if (Pearson[name, 1] < 0.05) 
            linear = append(linear, name)
        else if (Spearman[name, 1] < 0.05)
            mono = append(mono, name)
        else if (HHG[name, 1] < 0.05)
            nmono = append(nmono, name)
    }
}

# Antes de executar selecioanar pacientes do estagio I, carregar tabela
# com os niveis de expressao genica, e definir os laboratorios a
# serem utilizados
plot.gene.without.outliers <- function(gene) {
    data = list()
    selected_patients = patients[which(patients %in% sites)]
    WNT5A <- as.numeric(table["WNT5A", paste("X", names(selected_patients), sep="")])
    Y = as.numeric(table[gene, paste("X", names(selected_patients), sep="")])
    names(Y) = paste("X", names(selected_patients), sep="")

    label = sites[1]
    if (length(sites) > 1)
        for (i in 2:length(sites))
            label=paste(label, sites[i], sep=",")
    label = paste(label, " - ", sep="")

    for (site in sites) {
            samesite <- paste("X", names(selected_patients[which(selected_patients == site)]), sep="")
            pdf(".pdf")
            b = boxplot(Y[samesite])
            dev.off()
            index = which(!(Y %in% b$out))
            Y = Y[index]
            WNT5A = WNT5A[index]
    }
    
    x = data$x = log2(WNT5A)
    y = data$y = log2(Y)
    B <- bs(x, df=5)
    summary(fm1 <- lm(y ~ bs(x, df=5)))
    pdf(paste(paste(
        "~/Dropbox/Suzana/Dados/Gráficos/Sem outliers - ", gene, sep=label), 
        "pdf", sep="."))
    plot(data, col="black", xlab="WNT5A", ylab=gene, pch=20, lwd=4)
    ht = seq(min(data$x), max(data$x), length.out=220)
    lines(ht, predict(fm1, data.frame(x=ht)), col="red", lwd=2)
    dev.off()
}
