# ------------------------------------------------------------------------------

# Construcao da curva poder

# ------------------------------------------------------------------------------


# ================================ DEPENDENCIAS ================================ 

source("riemann.r")


# ============================ FUNCOES DA INTERFACE ============================ 

# Recebe:
# pvalues: matriz ds pvalores dos testes estatiscos realizados. Para cada metodo 
# utilizado nos testes, ha uma coluna correspondente na matriz e, para cada
# simulacao, ha uma linha correspondente.
# npoints: numero de pontos utilizados para a construcao do grafico.
#
# Constroi curva poder a partir dos pvalores recebidos.
power.plot <- function (pvalues, npoints = 51, title = "", legends = NULL) {
    ntests <- ncol(pvalues)
    n <- nrow(pvalues)
    r <- 1/(npoints - 1)
    
    alpha <- 0  
    
    RRs <- matrix(0, nrow = npoints, ncol = ntests)
    
    for (i in 1:npoints) {
        for (j in 1:ntests) { 
            #for (k in 1:n)
                #if (pvalues[k, j] < alpha)
                    #RRs[i, j] <- RRs[i, j] + 1
            RRs[i, j] <- length(which(pvalues[, j] < alpha)) 
            RRs[i, j] <- RRs[i, j]/n
        }
        alpha <- alpha + r
    }
    
    pdf(paste(title,".pdf", sep = ""))
    matplot(seq(0, 1, r), RRs, xlab = "Nível de significância do teste (TFP)",
            ylab = "Proporção de rejeição (TVP)", pch = 1, type = "l", lwd = 2, 
            col=2:(2 + ntests - 1), lty=1)
    title(title)
    lines(seq(0, 1, 0.01), seq(0, 1, 0.01), lty=2)
    if (length(legends) != 0)
        legend(x = "bottomright", legend = legends, cex=0.8, col=2:(2 + ntests - 1), 
               lty=1, lwd=2, bty="n");
    dev.off()
}

power.area <- function (file) {
    pvalues <- as.matrix(read.table(file))
    power.plot (pvalues, npoints = 51, title = , legends = NULL)
    pvalues <- as.numeric(readLines(file))
    f <- function (alpha) {
        return (length(which(pvalues < alpha))/length(pvalues)) 
    }
    return (riemann.sum(f, 0, 1))   
}
