# ------------------------------------------------------------------------------

# Construcao de curvas poder para a comparacao entre as medidas de Pearson,
# Spearman e Hoeffding

# ------------------------------------------------------------------------------


# ================================ DEPENDENCIAS ================================ 

source("test.r")
source("power.r")


# ============================= FUNCOES AUXILIARES ============================= 

# Recebe:
# n - tamanho da amostra a ser gerada
# Gera amostra de variável com distribuição uniforme no intervalo [-1, 1]
unif <- function (n) {
    return(runif(n, min = -1))
}

# ============================ FUNCOES DA INTERFACE ============================ 

# Controi curvas poder para a comparacao entre as medidas de Pearson, Spearman e
# Hoeffding, correlação de distância e HHG
simulate.all <- function (n = 30, methods = 
                      list("funcoes" = c(pearson.test, spearman.test, 
                           kendall.test, hoeffding.test, dcorrelation.test, 
                           hhg.test, mutualinfo.test), "legendas" = c("Pearson", 
                           "Spearman", "Kendall", "Hoeffding", "Dcov", "HHG",
                           "IM"))) {
   
  
    # Independente
    x <- seq(-3, 3, 6/(n - 1))
    simulate(x, title="Independente", methods = methods)
    # Linear vom ruido grande
    #x <- seq(-3, 3, 6/(n - 1))
    x = seq(-1.5, 1,  2.5/(n - 2))
    x[n] = 2.5
    simulate(x, association = 0.5*x, title = "Linear com ruido grande", methods = methods)
    # Quadratica
    x <- seq(-3, 3, 6/(n - 1))
    simulate(x, association = x*x, title = "Quadratica", methods = methods)
    # Quadratica com ruido grande
    x <- seq(-1.5, 1.5, 3/(n - 1))
    simulate(x, association = x*x, title = "Quadratica com ruido grande", methods = methods)
    # Exponencial
    #x <- seq(0, 3, 3/(n - 1))
    #simulate(x, association = x*x, title = "Semi-quadratica", methods = methods)
    x = seq(-30, 20, 50/(n-1))
    simulate(x, association = 0.01*exp(x), title = "Exponencial", methods = methods)
    
    # Seno
    x <- seq(0, 10, 10/(n - 1))
    simulate(x, association = 2*sin(x), noise = unif, title = "Seno", 
             methods = methods)
    # Bi-quadratica
    x <- seq(-2, 2, 4/(n - 1))
    simulate(x, association = -x*x*x*x, title = "Bi-quadratica", 
             methods = methods)
    # Cubica
    x <- seq(0.4, 1.6, 1.2/(n - 1))
    simulate(x, association = 30*(x - 0.5)*(x - 1)*(x - 1.5), title = "Cubica",
    methods = methods)
    # Gaussiana
    # x <- seq(-1, 1, 2/29)
    # simulate(x, association = exp(-x*x/2)/sqrt(2*pi), title = "Gaussiana")
    # Circunferencia
    y <- x <- vector(length = n)
    x[1:(n/2)] <- seq(-5, 5, 10/(n/2 - 1))
    x[(n/2 + 1):n] <- seq(5, -5, -10/(n/2 - 1))
    y[1:(n/2)] <- sqrt(25 - x[1:(n/2)]*x[1:(n/2)])
    y[(n/2 + 1):n] <- -sqrt(25 - x[(n/2 + 1):n]*x[(n/2 + s1):n])
    simulate(x, association = y, noise = unif, title = "Circunferencia", 
             methods = methods)
    # X
    y <- x <- vector(length = n)
    x[1:(n/2)] <- seq(-5, 5, 10/(n/2 - 1))
    x[(n/2 + 1):n] <- seq(5, -5, -10/(n/2 - 1))
    y[1:(n/2)] <- x[1:(n/2)]
    y[(n/2 + 1):n] <- -x[(n/2 + 1):n]
    simulate(x, association = y, noise = unif, title = "X", methods = methods)

    # Estudo dos efeitos da intrudocao de outliers -----------------------------
    
    # Independente
    x <- seq(-3, 3, 6/(n - 1))
    simulate(x, outliers.index = (min(n, n - as.integer(0.07*n) + 1)):n, 
             title = "Independente com outliers", 
    methods = methods)
    # Linear
    x <- seq(-3, 3, 6/(n - 1))
    simulate(x, outliers.index = (min(n, n - as.integer(0.07*n) + 1)):n, 
             association = 0.5*x, title = "Linear com outliers", 
             methods = methods)
    # Quadratica
    x <- seq(-2, 2, 4/(n - 1))       
    simulate(x, outliers.index = (min(n/2 + 1, n/2 - as.integer(0.07*n) + 2)):(n/2 + 1), 
             association = x*x, title = "Quadratica com outliers", 
             methods = methods)
    # Bi-quadratica
    x <- seq(-2, 2, 4/(n - 1))
    simulate(x, outliers.index = (min(n/2 + 1, n/2 - as.integer(0.07*n) + 2)):(n/2 + 1), 
             association = -x*x*x*x, title = "Bi-quadratica com outliers", 
             methods = methods)
    # Seno
    x <- seq(0, 10, 10/(n - 1))
    simulate(x, outliers.index = 1:max(1, (as.integer(0.07*n))), association = 2*sin(x), 
             noise = unif, title = "Seno com outliers", methods = methods) 
    # Gaussiana
    #x <- seq(-1, 1, 2/29)
    #simulate(x, outliers.index = 15:16, association = exp(-x*x/2)/sqrt(2*pi), 
    #         title = "Gaussiana com outliers")
    # Circunferencia
    # y <- x <- vector(length = 30)
    # x[1:15] <- seq(-5, 5, 10/14)
    # x[16:30] <- seq(5, -5, -10/14)
    # y[1:15] <- sqrt(25 - x[1:15]*x[1:15])
    # y[16:30] <- -sqrt(25 - x[16:30]*x[16:30])
    # simulate(x, outliers.index = 15:16, association = y, 
    #          title = "Circunferencia com outliers",
    # methods = methods)
    # # X
    # y <- x <- vector(length = 30)
    # x[1:15] <- seq(-5, 5, 10/14)
    # x[16:30] <- seq(5, -5, -10/14)
    # y[1:15] <- x[1:15]
    # y[16:30] <- -x[16:30]
    # simulate(x, outliers.index = 15:16, association = y, title = "X com outliers", 
    # methods = methods)
}

# Recebe:
# x: amostra
# times: numero de simulacoes a serem realizadas
# outliers.index: vetor das posicoes da amostra y em que serao inseridos 
# os outliers. No caso de nao haver outlier, seu valor e' NULL
# association: Vetor com os valores a partir dos quais sera gerada a amostra y
# (em geral, contem valores resultantes de uma funcao aplicada em x). Caso
# x e y sejam independentes, seu valor e' 0.
# noise: ruído aplicado à amostra y
# methods: lista com:
#   $funcoes - um vetor das funções de testes de independencia a serem
#   utilizadas  na simulacao
#   $legendas - legendas dos metodos aplicados na simulacao
#
# Constroi a curva poder a partir dos testes estatisticos realizados nas 
# simulacoes, utilizando-se as medidas de Pearson, Spearman e Hoeffding 
simulate <- function (x, times = 1000, outliers.index = NULL, 
                      association = 0, noise = rnorm, title="", methods = 
                      list("funcoes" = c(pearson.test, spearman.test, 
                      hoeffding.test, dcorrelation.test, hhg.test),
                      "legendas" = c("Pearson", "Spearman", "Hoeffding", 
                      "Dcov", "HHG"))) {
                      
    nmethods <- length(methods$funcoes)
    pvalues <- matrix(0, times, nmethods)
    
    n <- length(x)
    
    title <- paste(n, title,  sep=" - ")
    title <- paste("n", title,  sep="=")
    
    for (i in 1:times) {
        y <- association + noise(n)
        outliers.length <- length(outliers.index)
        if (outliers.length != 0) {
            outliers <- rnorm(outliers.length, 0, 100)
            for (j in 1:outliers.length)
                y[outliers.index[j]] <- outliers[j]
        }

        for (j in 1:nmethods) {
            pvalues[i, j] <- as.numeric(methods$funcoes[[j]](x, y)["p-valor"])
            write(pvalues[i,j], paste(title, methods$legendas[j], sep=" - "),  
                  append = TRUE)
        }
    }
    
    #write.table(methods$legendas, "oi")
    #write.table(pvalues, "oi", append = TRUE, row.names=FALSE)
       
    #power.plot(pvalues, title = title, legends = methods$legendas)
}

build.table <- function (directory) {
    index <- which(!file.info(paste(directory, dir(directory), sep = "/"))[2])
    i <- 0
    j <- 0
    rows <- vector()
    cols <- vector()
    for (file in dir(directory)[index]) {
        name <- strsplit(file, " - ")[[1]]
        i <- which(rows == name[2])[1]
        if (is.na(i)) i <- length(rows) + 1
        rows[i] <- name[2]
        j <- which(cols == name[3])[1]
        if (is.na(j)) j <- length(cols) + 1
        cols[j] <- name[3]
    }
    # TODO Provisório: 
    cols = c("Pearson", "Dcov", "Spearman", "Kendall", "Hoeffding", "HHG", "IM", "CIM")
    table <- matrix(NA, nrow = length(rows), ncol = length(cols))
    rownames(table) <- rows
    colnames(table) <- cols
    for (file in dir(directory)[index]) {
        area <- power.area(paste(directory, file, sep = "/"))
        name <- strsplit(file, " - ")[[1]]
        table[name[2], name[3]] <- area       
    }
    write.table(table, paste("Comparacao das curvas roc - ", name[1], sep=" "))
    print(table)
}
